@@ -67,16 +67,6 @@ if os.name == "nt":
                 return fname
         return None
 
-if os.name == "ce":
-    # search path according to MSDN:
-    # - absolute path specified by filename
-    # - The .exe launch directory
-    # - the Windows directory
-    # - ROM dll files (where are they?)
-    # - OEM specified search path: HKLM\Loader\SystemPath
-    def find_library(name):
-        return name
-
 if os.name == "posix" and sys.platform == "darwin":
     from ctypes.macholib.dyld import dyld_find as _dyld_find
     def find_library(name):
@@ -102,8 +92,6 @@ elif os.name == "posix":
         expr = os.fsencode(r'[^\(\)\s]*lib%s\.[^\(\)\s]*' % re.escape(name))
 
         c_compiler = shutil.which('gcc')
-        if not c_compiler:
-            c_compiler = shutil.which('clang')
         if not c_compiler:
             c_compiler = shutil.which('cc')
         if not c_compiler:
@@ -112,7 +100,7 @@ elif os.name == "posix":
 
         temp = tempfile.NamedTemporaryFile()
         try:
-            args = [c_compiler, '-Wl,-R/usr/pkg/lib', '-L/usr/pkg/lib', '-Wl,-R/usr/X11R7/lib', '-L/usr/X11R7/lib', '-Wl,-t', '-o', temp.name, '-l' + name]
+            args = [c_compiler, '-Wl,-R/usr/pkg/lib', '-L/usr/pkg/lib', '-Wl,-t', '-o', temp.name, '-l' + name]
 
             env = dict(os.environ)
             env['LC_ALL'] = 'C'
@@ -273,8 +261,8 @@ elif os.name == "posix":
             abi_type = mach_map.get(machine, 'libc6')
 
             # XXX assuming GLIBC's ldconfig (with option -p)
-            regex = os.fsencode(
-                '\s+(lib%s\.[^\s]+)\s+\(%s' % (re.escape(name), abi_type))
+            regex = r'\s+(lib%s\.[^\s]+)\s+\(%s'
+            regex = os.fsencode(regex % (re.escape(name), abi_type))
             try:
                 with subprocess.Popen(['/sbin/ldconfig', '-p'],
                                       stdin=subprocess.DEVNULL,
